import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { bindActionCreators } from 'redux';
import classNames from 'classnames';
import { debounce } from 'lodash';
import { autobind } from 'core-decorators';
import { MentionsInput, Mention } from 'react-mentions';
import { addFiles, removeFile, setUserTyping, submitActivity, storeActivityText } from '@ciscospark/redux-module-activity';
import { searchForUser } from '@ciscospark/redux-module-search';
import PresenceAvatar from '@ciscospark/react-container-presence-avatar';
import FileStagingArea from '@ciscospark/react-component-file-staging-area';
import { constructFiles } from '@ciscospark/react-component-utils';
import bowser from 'bowser';
import { Icon } from '@momentum-ui/react';
import { fromJS } from 'immutable';

function styleInject(css, ref) {
  if (ref === void 0) ref = {};
  var insertAt = ref.insertAt;

  if (!css || typeof document === 'undefined') {
    return;
  }

  var head = document.head || document.getElementsByTagName('head')[0];
  var style = document.createElement('style');
  style.type = 'text/css';

  if (insertAt === 'top') {
    if (head.firstChild) {
      head.insertBefore(style, head.firstChild);
    } else {
      head.appendChild(style);
    }
  } else {
    head.appendChild(style);
  }

  if (style.styleSheet) {
    style.styleSheet.cssText = css;
  } else {
    style.appendChild(document.createTextNode(css));
  }
}

var css = ".react-container-message-composer__buttonsArea__LmJ1dHRv {\n  padding: 10px 10px 0;\n}\n\n.react-container-message-composer__fileInput__LmJ1dHRv {\n  display: none;\n  width: 32px;\n  height: 32px;\n  overflow: hidden;\n  opacity: 0;\n}\n";
var styles = { "buttonsArea": "react-container-message-composer__buttonsArea__LmJ1dHRv", "fileInput": "react-container-message-composer__fileInput__LmJ1dHRv" };
styleInject(css);

function ComposerButtons(_ref) {
  var onAttachFile = _ref.onAttachFile,
      composerActions = _ref.composerActions;

  function handleFileIconClick(event) {
    if (event.type === 'click' || event.type === 'keydown' && (event.keyCode === 13 || event.keyCode === 32)) {
      if (!bowser.safari) {
        document.getElementById('messageFileUpload').click();
      }
    }
  }

  return composerActions.attachFiles && React.createElement(
    'div',
    { className: classNames('ciscospark-message-composer-buttons', styles.buttonsArea) },
    React.createElement(
      'label',
      { htmlFor: 'messageFileUpload' },
      React.createElement(Icon, {
        ariaLabel: 'Attach Files',
        name: 'attachment_16',
        onClick: handleFileIconClick
      }),
      React.createElement('input', {
        className: classNames('ciscospark-file-input', styles.fileInput),
        id: 'messageFileUpload',
        multiple: 'multiple',
        onChange: onAttachFile,
        type: 'file'
      })
    )
  ) || null;
}

ComposerButtons.propTypes = {
  onAttachFile: PropTypes.func
};

ComposerButtons.defaultProps = {
  onAttachFile: function onAttachFile() {}
};

var UPDATE_TEXTAREA_FOCUS = 'UPDATE_TEXTAREA_FOCUS';

function blurTextArea() {
  return {
    type: UPDATE_TEXTAREA_FOCUS,
    payload: {
      hasTextAreaFocus: false
    }
  };
}

function focusTextArea() {
  return {
    type: UPDATE_TEXTAREA_FOCUS,
    payload: {
      hasTextAreaFocus: true
    }
  };
}

var css$1 = ".react-container-message-composer__messageComposer__Lm1lc3Nh {\n  position: relative;\n  height: auto;\n  border-top: 1px solid #ebebeb;\n  transition: all 0.25s cubic-bezier(0.22, 0.61, 0.36, 1);\n}\n\n.react-container-message-composer__hasFocus__Lm1lc3Nh {\n  border-top-color: #7ddef0;\n}\n\n.react-container-message-composer__addFileContainer__Lm1lc3Nh {\n  position: absolute;\n  top: 12px;\n  left: 20px;\n}\n\n.react-container-message-composer__textAreaContainer__Lm1lc3Nh {\n  background-color: #f5f5f5;\n  padding: 10px;\n}\n\n.react-container-message-composer__textarea__Lm1lc3Nh {\n  background-color: #f5f5f5;\n}\n\n.react-container-message-composer__textarea__Lm1lc3Nh::placeholder {\n  overflow: hidden;\n  color: #858688;\n  text-overflow: ellipsis;\n  white-space: nowrap;\n}\n\n.react-container-message-composer__textareaContainerBig__Lm1lc3Nh {\n  height: 150px;\n}\n";
var styles$1 = { "messageComposer": "react-container-message-composer__messageComposer__Lm1lc3Nh", "hasFocus": "react-container-message-composer__hasFocus__Lm1lc3Nh", "addFileContainer": "react-container-message-composer__addFileContainer__Lm1lc3Nh", "textAreaContainer": "react-container-message-composer__textAreaContainer__Lm1lc3Nh", "textarea": "react-container-message-composer__textarea__Lm1lc3Nh", "textareaContainerBig": "react-container-message-composer__textareaContainerBig__Lm1lc3Nh" };
styleInject(css$1);

var css$2 = "/* Using !important here to override inline styles from react-mentions */\n\n.react-container-message-composer__mentions__LyogVXNp {\n  position: static;\n  display: flex;\n  flex-direction: column;\n  margin: 0;\n  font-size: 14px;\n}\n\n.react-container-message-composer__mentions__control__LyogVXNp {\n  position: relative;\n  max-height: 100px;\n}\n\n.react-container-message-composer__mentions__input__LyogVXNp {\n  display: block;\n  margin: 0;\n  font-family: inherit;\n  font-size: 14px;\n  font-weight: 200;\n  background: transparent;\n  border: none;\n  outline: none;\n  box-sizing: border-box;\n}\n\n.react-container-message-composer__mentions__input__LyogVXNp:focus {\n  outline: none;\n  box-shadow: none;\n}\n\n.react-container-message-composer__mentions--singleLine__LyogVXNp .react-container-message-composer__mentions__control__LyogVXNp {\n  display: inline-block;\n  width: 130px;\n}\n\n.react-container-message-composer__mentions--singleLine__LyogVXNp .react-container-message-composer__mentions__higlighter__LyogVXNp {\n  padding: 1px;\n  border: 2px inset transparent;\n}\n\n.react-container-message-composer__mentions--singleLine__LyogVXNp .react-container-message-composer__mentions__input__LyogVXNp {\n  padding: 1px;\n  border: 0;\n}\n\n.react-container-message-composer__mentions--multiLine__LyogVXNp .react-container-message-composer__mentions__highlighter__LyogVXNp {\n  padding: 2px;\n  margin: 0;\n}\n\n.react-container-message-composer__mentions--multiLine__LyogVXNp .react-container-message-composer__mentions__input__LyogVXNp {\n  /* stylelint-disable-next-line declaration-no-important */\n  overflow-y: auto !important;\n  resize: none;\n}\n\n.react-container-message-composer__mentions--multiLine__LyogVXNp .react-container-message-composer__mentions__input__LyogVXNp::placeholder {\n  overflow: hidden;\n  color: #858688;\n  text-overflow: ellipsis;\n  white-space: nowrap;\n}\n\n.react-container-message-composer__mentions__LyogVXNp .react-container-message-composer__mentions__suggestions__LyogVXNp {\n  /* stylelint-disable declaration-no-important */\n  position: static !important;\n  order: -1;\n  left: 0 !important;\n  margin-top: 0 !important;\n}\n\n.react-container-message-composer__mentions__suggestions__list__LyogVXNp {\n  max-height: 85px;\n  overflow-y: auto;\n  font-size: 14px;\n  background-color: #fff;\n}\n\n.react-container-message-composer__mentions__suggestions__item__LyogVXNp {\n  padding: 5px 15px;\n  border-bottom: 1px solid rgba(0, 0, 0, 0.15);\n}\n\n.react-container-message-composer__content__LyogVXNp {\n  display: flex;\n}\n\n.react-container-message-composer__avatar__LyogVXNp {\n  flex: 0 0 auto;\n  width: 24px;\n  height: 24px;\n  margin-right: 10px;\n}\n\n.react-container-message-composer__highlightedDisplay__LyogVXNp {\n  flex: 1 1 auto;\n  display: flex;\n  align-items: center;\n}\n\n.react-container-message-composer__mentions__suggestions__item--focused__LyogVXNp {\n  background-color: #ececed;\n}\n\n.react-container-message-composer__mentions__mention__LyogVXNp {\n  background-color: #ececed;\n}\n";
var mentionStyles = { "mentions": "react-container-message-composer__mentions__LyogVXNp", "mentions__control": "react-container-message-composer__mentions__control__LyogVXNp", "mentions__input": "react-container-message-composer__mentions__input__LyogVXNp", "mentions--singleLine": "react-container-message-composer__mentions--singleLine__LyogVXNp", "mentions__higlighter": "react-container-message-composer__mentions__higlighter__LyogVXNp", "mentions--multiLine": "react-container-message-composer__mentions--multiLine__LyogVXNp", "mentions__highlighter": "react-container-message-composer__mentions__highlighter__LyogVXNp", "mentions__suggestions": "react-container-message-composer__mentions__suggestions__LyogVXNp", "mentions__suggestions__list": "react-container-message-composer__mentions__suggestions__list__LyogVXNp", "mentions__suggestions__item": "react-container-message-composer__mentions__suggestions__item__LyogVXNp", "content": "react-container-message-composer__content__LyogVXNp", "avatar": "react-container-message-composer__avatar__LyogVXNp", "highlightedDisplay": "react-container-message-composer__highlightedDisplay__LyogVXNp", "mentions__suggestions__item--focused": "react-container-message-composer__mentions__suggestions__item--focused__LyogVXNp", "mentions__mention": "react-container-message-composer__mentions__mention__LyogVXNp" };
styleInject(css$2);

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
};

var jsx = function () {
  var REACT_ELEMENT_TYPE = typeof Symbol === "function" && Symbol.for && Symbol.for("react.element") || 0xeac7;
  return function createRawReactElement(type, props, key, children) {
    var defaultProps = type && type.defaultProps;
    var childrenLength = arguments.length - 3;

    if (!props && childrenLength !== 0) {
      props = {};
    }

    if (props && defaultProps) {
      for (var propName in defaultProps) {
        if (props[propName] === void 0) {
          props[propName] = defaultProps[propName];
        }
      }
    } else if (!props) {
      props = defaultProps || {};
    }

    if (childrenLength === 1) {
      props.children = children;
    } else if (childrenLength > 1) {
      var childArray = Array(childrenLength);

      for (var i = 0; i < childrenLength; i++) {
        childArray[i] = arguments[i + 3];
      }

      props.children = childArray;
    }

    return {
      $$typeof: REACT_ELEMENT_TYPE,
      type: type,
      key: key === undefined ? null : '' + key,
      ref: null,
      props: props,
      _owner: null
    };
  };
}();

var asyncIterator = function (iterable) {
  if (typeof Symbol === "function") {
    if (Symbol.asyncIterator) {
      var method = iterable[Symbol.asyncIterator];
      if (method != null) return method.call(iterable);
    }

    if (Symbol.iterator) {
      return iterable[Symbol.iterator]();
    }
  }

  throw new TypeError("Object is not async iterable");
};

var asyncGenerator = function () {
  function AwaitValue(value) {
    this.value = value;
  }

  function AsyncGenerator(gen) {
    var front, back;

    function send(key, arg) {
      return new Promise(function (resolve, reject) {
        var request = {
          key: key,
          arg: arg,
          resolve: resolve,
          reject: reject,
          next: null
        };

        if (back) {
          back = back.next = request;
        } else {
          front = back = request;
          resume(key, arg);
        }
      });
    }

    function resume(key, arg) {
      try {
        var result = gen[key](arg);
        var value = result.value;

        if (value instanceof AwaitValue) {
          Promise.resolve(value.value).then(function (arg) {
            resume("next", arg);
          }, function (arg) {
            resume("throw", arg);
          });
        } else {
          settle(result.done ? "return" : "normal", result.value);
        }
      } catch (err) {
        settle("throw", err);
      }
    }

    function settle(type, value) {
      switch (type) {
        case "return":
          front.resolve({
            value: value,
            done: true
          });
          break;

        case "throw":
          front.reject(value);
          break;

        default:
          front.resolve({
            value: value,
            done: false
          });
          break;
      }

      front = front.next;

      if (front) {
        resume(front.key, front.arg);
      } else {
        back = null;
      }
    }

    this._invoke = send;

    if (typeof gen.return !== "function") {
      this.return = undefined;
    }
  }

  if (typeof Symbol === "function" && Symbol.asyncIterator) {
    AsyncGenerator.prototype[Symbol.asyncIterator] = function () {
      return this;
    };
  }

  AsyncGenerator.prototype.next = function (arg) {
    return this._invoke("next", arg);
  };

  AsyncGenerator.prototype.throw = function (arg) {
    return this._invoke("throw", arg);
  };

  AsyncGenerator.prototype.return = function (arg) {
    return this._invoke("return", arg);
  };

  return {
    wrap: function (fn) {
      return function () {
        return new AsyncGenerator(fn.apply(this, arguments));
      };
    },
    await: function (value) {
      return new AwaitValue(value);
    }
  };
}();

var asyncGeneratorDelegate = function (inner, awaitWrap) {
  var iter = {},
      waiting = false;

  function pump(key, value) {
    waiting = true;
    value = new Promise(function (resolve) {
      resolve(inner[key](value));
    });
    return {
      done: false,
      value: awaitWrap(value)
    };
  }

  if (typeof Symbol === "function" && Symbol.iterator) {
    iter[Symbol.iterator] = function () {
      return this;
    };
  }

  iter.next = function (value) {
    if (waiting) {
      waiting = false;
      return value;
    }

    return pump("next", value);
  };

  if (typeof inner.throw === "function") {
    iter.throw = function (value) {
      if (waiting) {
        waiting = false;
        throw value;
      }

      return pump("throw", value);
    };
  }

  if (typeof inner.return === "function") {
    iter.return = function (value) {
      return pump("return", value);
    };
  }

  return iter;
};

var asyncToGenerator = function (fn) {
  return function () {
    var gen = fn.apply(this, arguments);
    return new Promise(function (resolve, reject) {
      function step(key, arg) {
        try {
          var info = gen[key](arg);
          var value = info.value;
        } catch (error) {
          reject(error);
          return;
        }

        if (info.done) {
          resolve(value);
        } else {
          return Promise.resolve(value).then(function (value) {
            step("next", value);
          }, function (err) {
            step("throw", err);
          });
        }
      }

      return step("next");
    });
  };
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

var defineEnumerableProperties = function (obj, descs) {
  for (var key in descs) {
    var desc = descs[key];
    desc.configurable = desc.enumerable = true;
    if ("value" in desc) desc.writable = true;
    Object.defineProperty(obj, key, desc);
  }

  return obj;
};

var defaults = function (obj, defaults) {
  var keys = Object.getOwnPropertyNames(defaults);

  for (var i = 0; i < keys.length; i++) {
    var key = keys[i];
    var value = Object.getOwnPropertyDescriptor(defaults, key);

    if (value && value.configurable && obj[key] === undefined) {
      Object.defineProperty(obj, key, value);
    }
  }

  return obj;
};

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var get = function get(object, property, receiver) {
  if (object === null) object = Function.prototype;
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent === null) {
      return undefined;
    } else {
      return get(parent, property, receiver);
    }
  } else if ("value" in desc) {
    return desc.value;
  } else {
    var getter = desc.get;

    if (getter === undefined) {
      return undefined;
    }

    return getter.call(receiver);
  }
};

var inherits = function (subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      enumerable: false,
      writable: true,
      configurable: true
    }
  });
  if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
};

var _instanceof = function (left, right) {
  if (right != null && typeof Symbol !== "undefined" && right[Symbol.hasInstance]) {
    return right[Symbol.hasInstance](left);
  } else {
    return left instanceof right;
  }
};

var interopRequireDefault = function (obj) {
  return obj && obj.__esModule ? obj : {
    default: obj
  };
};

var interopRequireWildcard = function (obj) {
  if (obj && obj.__esModule) {
    return obj;
  } else {
    var newObj = {};

    if (obj != null) {
      for (var key in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key];
      }
    }

    newObj.default = obj;
    return newObj;
  }
};

var newArrowCheck = function (innerThis, boundThis) {
  if (innerThis !== boundThis) {
    throw new TypeError("Cannot instantiate an arrow function");
  }
};

var objectDestructuringEmpty = function (obj) {
  if (obj == null) throw new TypeError("Cannot destructure undefined");
};

var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

var possibleConstructorReturn = function (self, call) {
  if (!self) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return call && (typeof call === "object" || typeof call === "function") ? call : self;
};

var selfGlobal = typeof global === "undefined" ? self : global;

var set = function set(object, property, value, receiver) {
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent !== null) {
      set(parent, property, value, receiver);
    }
  } else if ("value" in desc && desc.writable) {
    desc.value = value;
  } else {
    var setter = desc.set;

    if (setter !== undefined) {
      setter.call(receiver, value);
    }
  }

  return value;
};

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

var slicedToArrayLoose = function (arr, i) {
  if (Array.isArray(arr)) {
    return arr;
  } else if (Symbol.iterator in Object(arr)) {
    var _arr = [];

    for (var _iterator = arr[Symbol.iterator](), _step; !(_step = _iterator.next()).done;) {
      _arr.push(_step.value);

      if (i && _arr.length === i) break;
    }

    return _arr;
  } else {
    throw new TypeError("Invalid attempt to destructure non-iterable instance");
  }
};

var taggedTemplateLiteral = function (strings, raw) {
  return Object.freeze(Object.defineProperties(strings, {
    raw: {
      value: Object.freeze(raw)
    }
  }));
};

var taggedTemplateLiteralLoose = function (strings, raw) {
  strings.raw = raw;
  return strings;
};

var temporalRef = function (val, name, undef) {
  if (val === undef) {
    throw new ReferenceError(name + " is not defined - temporal dead zone");
  } else {
    return val;
  }
};

var temporalUndefined = {};

var toArray = function (arr) {
  return Array.isArray(arr) ? arr : Array.from(arr);
};

var toConsumableArray = function (arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

    return arr2;
  } else {
    return Array.from(arr);
  }
};

var babelHelpers = /*#__PURE__*/Object.freeze({
  jsx: jsx,
  asyncIterator: asyncIterator,
  asyncGenerator: asyncGenerator,
  asyncGeneratorDelegate: asyncGeneratorDelegate,
  asyncToGenerator: asyncToGenerator,
  classCallCheck: classCallCheck,
  createClass: createClass,
  defineEnumerableProperties: defineEnumerableProperties,
  defaults: defaults,
  defineProperty: defineProperty,
  get: get,
  inherits: inherits,
  interopRequireDefault: interopRequireDefault,
  interopRequireWildcard: interopRequireWildcard,
  newArrowCheck: newArrowCheck,
  objectDestructuringEmpty: objectDestructuringEmpty,
  objectWithoutProperties: objectWithoutProperties,
  possibleConstructorReturn: possibleConstructorReturn,
  selfGlobal: selfGlobal,
  set: set,
  slicedToArray: slicedToArray,
  slicedToArrayLoose: slicedToArrayLoose,
  taggedTemplateLiteral: taggedTemplateLiteral,
  taggedTemplateLiteralLoose: taggedTemplateLiteralLoose,
  temporalRef: temporalRef,
  temporalUndefined: temporalUndefined,
  toArray: toArray,
  toConsumableArray: toConsumableArray,
  typeof: _typeof,
  extends: _extends,
  instanceof: _instanceof
});

var _class;

function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) {
  var desc = {};
  Object['ke' + 'ys'](descriptor).forEach(function (key) {
    desc[key] = descriptor[key];
  });
  desc.enumerable = !!desc.enumerable;
  desc.configurable = !!desc.configurable;

  if ('value' in desc || desc.initializer) {
    desc.writable = true;
  }

  desc = decorators.slice().reverse().reduce(function (desc, decorator) {
    return decorator(target, property, desc) || desc;
  }, desc);

  if (context && desc.initializer !== void 0) {
    desc.value = desc.initializer ? desc.initializer.call(context) : void 0;
    desc.initializer = undefined;
  }

  if (desc.initializer === void 0) {
    Object['define' + 'Property'](target, property, desc);
    desc = null;
  }

  return desc;
}

var injectedPropTypes = {
  activity: PropTypes.object.isRequired,
  avatar: PropTypes.object.isRequired,
  conversation: PropTypes.object.isRequired,
  sparkInstance: PropTypes.object.isRequired,
  users: PropTypes.object.isRequired,
  currentUser: PropTypes.object.isRequired,
  messageComposer: PropTypes.object.isRequired,
  addFiles: PropTypes.func.isRequired,
  blurTextArea: PropTypes.func.isRequired,
  focusTextArea: PropTypes.func.isRequired,
  removeFile: PropTypes.func.isRequired,
  setUserTyping: PropTypes.func.isRequired,
  submitActivity: PropTypes.func.isRequired,
  searchForUser: PropTypes.func.isRequired,
  storeActivityText: PropTypes.func.isRequired
};

var propTypes = _extends({
  composerActions: PropTypes.shape({
    attachFiles: PropTypes.bool
  }).isRequired,
  onSubmit: PropTypes.func.isRequired,
  placeholder: PropTypes.string,
  showMentions: PropTypes.bool
}, injectedPropTypes);

var defaultProps = {
  placeholder: '',
  showMentions: false
};

var MessageComposer = (_class = function (_Component) {
  inherits(MessageComposer, _Component);

  function MessageComposer(props) {
    classCallCheck(this, MessageComposer);

    var _this = possibleConstructorReturn(this, (MessageComposer.__proto__ || Object.getPrototypeOf(MessageComposer)).call(this, props));

    _this.timerId = 0;
    _this.setUserStartTyping = debounce(_this.setUserStartTyping, 500, { leading: true, trailing: false });
    return _this;
  }

  createClass(MessageComposer, [{
    key: 'shouldComponentUpdate',
    value: function shouldComponentUpdate(nextProps) {
      var props = this.props;


      return props.activity !== nextProps.activity || props.placeholder !== nextProps.placeholder;
    }
  }, {
    key: 'setUserTyping',
    value: function setUserTyping(state) {
      var props = this.props;
      var conversation = props.conversation,
          sparkInstance = props.sparkInstance;


      props.setUserTyping(state, conversation, sparkInstance);
    }
  }, {
    key: 'setUserStopTyping',
    value: function setUserStopTyping() {
      this.setUserTyping(false);
    }
  }, {
    key: 'setUserStartTyping',
    value: function setUserStartTyping() {
      this.setUserTyping(true);
    }
  }, {
    key: 'handleTextChange',
    value: function handleTextChange(e, newValue, newValuePlainText) {
      var props = this.props;
      var value = e.target.value;


      if (newValue && newValue.length) {
        value = newValue;
      }
      props.storeActivityText(value, newValuePlainText);

      clearTimeout(this.timerId);
      if (value === '') {
        this.setUserTyping(false);
      } else {
        if (!props.activity.getIn(['status', 'isTyping'])) {
          this.setUserStartTyping();
        }
        this.timerId = setTimeout(this.setUserStopTyping, 3000);
      }
    }
  }, {
    key: 'handleKeyDown',
    value: function handleKeyDown(e) {
      if (e.keyCode === 13 && !e.shiftKey && !e.altKey && !e.ctrlKey && !e.metaKey) {
        this.handleSubmit();
        e.preventDefault();
      }
    }
  }, {
    key: 'handleSubmit',
    value: function handleSubmit() {
      var props = this.props;
      var activity = props.activity,
          conversation = props.conversation,
          sparkInstance = props.sparkInstance,
          currentUser = props.currentUser,
          onSubmit = props.onSubmit;


      if (activity.get('text').length === 0 && activity.get('files').size === 0) {
        return;
      }
      this.setUserTyping(false);
      props.submitActivity(conversation, activity, currentUser, sparkInstance);

      if (typeof onSubmit === 'function') {
        onSubmit();
      }
    }
  }, {
    key: 'handleTextAreaBlur',
    value: function handleTextAreaBlur() {
      var props = this.props;
      var conversation = props.conversation,
          sparkInstance = props.sparkInstance;


      props.blurTextArea();
      props.setUserTyping(false, conversation, sparkInstance);
    }
  }, {
    key: 'handleTextAreaFocus',
    value: function handleTextAreaFocus() {
      this.props.focusTextArea();
    }
  }, {
    key: 'handleAddFile',
    value: function handleAddFile(e) {
      e.stopPropagation();
      e.preventDefault();

      if (e.target.files.length) {
        var props = this.props;
        var activity = props.activity,
            conversation = props.conversation,
            sparkInstance = props.sparkInstance;


        var files = constructFiles(e.target.files);

        props.addFiles(conversation, activity, files, sparkInstance);

        // Clear the value of the input so the same file can be added again.
        e.target.value = '';
      }
    }
  }, {
    key: 'handleFileRemove',
    value: function handleFileRemove(id) {
      var props = this.props;


      props.removeFile(id, props.activity);
    }
  }, {
    key: 'searchForMention',
    value: function searchForMention(term) {
      var _props = this.props,
          avatar = _props.avatar,
          conversation = _props.conversation,
          currentUser = _props.currentUser;

      var participants = [];

      if (conversation.has('participants') && conversation.get('participants').count()) {
        conversation.get('participants').toJS().forEach(function (p) {
          if (currentUser.id !== p.id) {
            participants.push({
              display: p.displayName,
              id: p.id,
              avatarUrl: avatar.getIn(['items', p.id])
            });
          }
        });

        if (term) {
          return participants.filter(function (p) {
            return p.display.toLowerCase().includes(term.toLowerCase());
          });
        }
      }

      return participants;
    }
  }, {
    key: 'render',
    value: function render() {
      var text = void 0;
      var props = this.props;
      var _props2 = this.props,
          activity = _props2.activity,
          composerActions = _props2.composerActions,
          messageComposer = _props2.messageComposer,
          placeholder = _props2.placeholder,
          conversation = _props2.conversation;


      var files = activity.get('files');

      if (activity && activity.has('text')) {
        text = activity.get('text');
      }

      var textAreaFocusStyle = messageComposer.getIn(['status', 'hasTextAreaFocus']) ? styles$1.hasFocus : '';
      var mentionMarkup = '@{__display__}|__id__|';

      // Only show mentions if this is not a one on one convo
      var showMentions = props.showMentions && !conversation.getIn(['status', 'isOneOnOne']);
      var getData = showMentions ? this.searchForMention : function () {};

      return React.createElement(
        'div',
        { className: classNames('ciscospark-message-composer', styles$1.messageComposer, textAreaFocusStyle) },
        React.createElement(ComposerButtons, {
          composerActions: composerActions,
          onAttachFile: this.handleAddFile
        }),
        files && files.count() > 0 && React.createElement(FileStagingArea, {
          files: files,
          onFileRemove: this.handleFileRemove
        }),
        React.createElement(
          'div',
          { className: classNames('ciscospark-textarea-container', styles$1.textAreaContainer) },
          React.createElement(
            MentionsInput,
            {
              classNames: mentionStyles,
              markup: mentionMarkup,
              onBlur: this.handleTextAreaBlur,
              onChange: this.handleTextChange,
              onFocus: this.handleTextAreaFocus,
              onKeyDown: this.handleKeyDown,
              onSubmit: this.handleSubmit,
              placeholder: placeholder,
              rows: 1,
              value: text
            },
            React.createElement(Mention, {
              className: mentionStyles.mentions__mention,
              data: getData,
              renderSuggestion: MessageComposer.renderSuggestion,
              trigger: '@'
            })
          )
        )
      );
    }
  }], [{
    key: 'renderSuggestion',
    value: function renderSuggestion(entry, search, highlightedDisplay) {
      return React.createElement(
        'div',
        { className: mentionStyles.content },
        React.createElement(
          'div',
          { className: mentionStyles.avatar },
          React.createElement(PresenceAvatar, { avatarId: entry.id, name: entry.display, size: 24 })
        ),
        React.createElement(
          'div',
          { className: mentionStyles.highlightedDisplay },
          highlightedDisplay
        )
      );
    }
  }]);
  return MessageComposer;
}(Component), (_applyDecoratedDescriptor(_class.prototype, 'setUserTyping', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'setUserTyping'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'setUserStopTyping', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'setUserStopTyping'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'handleTextChange', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'handleTextChange'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'handleKeyDown', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'handleKeyDown'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'handleSubmit', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'handleSubmit'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'handleTextAreaBlur', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'handleTextAreaBlur'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'handleTextAreaFocus', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'handleTextAreaFocus'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'handleAddFile', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'handleAddFile'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'handleFileRemove', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'handleFileRemove'), _class.prototype), _applyDecoratedDescriptor(_class.prototype, 'searchForMention', [autobind], Object.getOwnPropertyDescriptor(_class.prototype, 'searchForMention'), _class.prototype)), _class);

MessageComposer.propTypes = propTypes;
MessageComposer.defaultProps = defaultProps;

function mapStateToProps(state, ownProps) {
  return {
    activity: state.activity,
    avatar: state.avatar,
    conversation: state.conversation,
    sparkInstance: ownProps.sparkInstance || state.spark.get('spark'),
    users: state.users,
    messageComposer: state.messageComposer,
    currentUser: state.users.getIn(['byId', state.users.get('currentUserId')])
  };
}

var container = connect(mapStateToProps, function (dispatch) {
  return bindActionCreators({
    addFiles: addFiles,
    blurTextArea: blurTextArea,
    focusTextArea: focusTextArea,
    removeFile: removeFile,
    setUserTyping: setUserTyping,
    submitActivity: submitActivity,
    searchForUser: searchForUser,
    storeActivityText: storeActivityText
  }, dispatch);
})(MessageComposer);

var initialState = fromJS({
  status: {
    hasTextAreaFocus: false
  }
});

function reducer() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
  var action = arguments[1];

  switch (action.type) {
    case UPDATE_TEXTAREA_FOCUS:
      return state.setIn(['status', 'hasTextAreaFocus'], action.payload.state);
    default:
      return state;
  }
}

export default container;
export { MessageComposer, UPDATE_TEXTAREA_FOCUS, blurTextArea, focusTextArea, reducer };
//# sourceMappingURL=index.js.map
